<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Imports\AngsuransImport;
use App\Models\User;
use App\Models\Angsuran;
use App\Models\PotonganInternal;
use App\Models\JenisPotongan;
use Carbon\Carbon;
use DataTables;
use Session;
use Illuminate\Support\Facades\DB;

class AngsuranController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function list(Request $request)
    {
        $instansi = auth()->user()->instansi;
        $jenispotongans = JenisPotongan::all();
        if($instansi && auth()->user()->menuroles == 'instansi'){
            $jenispotongans = $instansi->potongantypes;
        }
        if(auth()->user()->menuroles == 'pegawai'){
            $pegawais = User::where('id', auth()->user()->id)->get();
        } else {
            $pegawais = User::where('menuroles', 'pegawai')->get();
        }
        // $filterTahun = Session::get('potongan_filter_tahun', null);
        // if(!$filterTahun && $request['tahun']){
            $filterTahun = $request['tahun'];
        // }
        // $filterBulan = Session::get('potongan_filter_bulan', null);
        // if(!$filterBulan && $request['bulan']){
            $filterBulan = $request['bulan'];
        // }
        // $filterJenis = Session::get('potongan_filter_jenis', null);
        // if(!$filterJenis && $request['jenis']){
            $filterJenis = $request['jenis'];
        // }
        // $filterPegawai = Session::get('potongan_filter_pegawai', null);
        // if(!$filterPegawai && $request['pegawai']){
            $filterPegawai = $request['pegawai']; 
        // }
        $potongans = Angsuran::with(['pegawai', 'jenis']);
        $potonganInternals = PotonganInternal::with(['pegawai', 'jenis']);
        if($filterTahun){
            $potongans = $potongans->where('tahun', $filterTahun);
            $potonganInternals = $potonganInternals->where('tahun', $filterTahun);
            Session::put('potongan_filter_tahun', $filterTahun); 
        }
        if($filterBulan){
            $potongans = $potongans->where('bulan', $filterBulan);
            $potonganInternals = $potonganInternals->where('bulan', $filterBulan);
            Session::put('potongan_filter_bulan', $filterBulan);
        }
        if($filterJenis){
            $potongans = $potongans->where('potongan_type_id', $filterJenis);
            $potonganInternals = $potonganInternals->where('potongan_type_id', $filterJenis);
            Session::put('potongan_filter_jenis', $filterJenis);
        }
        if($filterPegawai){
            $potongans = $potongans->where('pegawai_id', $filterPegawai);
            $potonganInternals = $potonganInternals->where('pegawai_id', $filterPegawai);
            Session::put('potongan_filter_pegawai', $filterPegawai);
        }
        $potongans = $potongans->select([
            DB::raw('"angsuran" as tipe'),
            'id',
            'bulan',
            'tahun',
            'pegawai_id',
            'potongan_type_id',
            'angsuran_total as jumlah',
            DB::raw('concat("Pembayaran angsuran dengan no surat perjanjian: ", surat_dasar_pemotongan, " Cicilan ke: ", angsuran_cicilan_ke, " Angsuran Pokok: ", angsuran_pokok, " Angsuran Bunga: ", angsuran_bunga, " Sisa Pinjaman: ", saldo_akhir) as keterangan')
        ]);
        // $potongans = $potongans->get();
        $potonganInternals = $potonganInternals->select([
            DB::raw('"potongan" as tipe'),
            'id',
            'bulan',
            'tahun',
            'pegawai_id',
            'potongan_type_id',
            'nominal as jumlah',
            DB::raw('"-" as keterangan')
        ]);
        // $potonganInternals = $potonganInternals->get();
        $potongans = $potongans->union($potonganInternals)->get();
        if($request->ajax()){
            return DataTables::of($potongans)
                ->addIndexColumn()
                ->addColumn('bulan', function($row){
                    return config('constants.bulans.' . $row->bulan);
                })
                ->addColumn('type', function($row){
                    return $row->jenis->name;
                })
                ->addColumn('pegawai', function($row){
                    $content = '<p class="my-0">'.
                        '<a href="'.route('pegawai.show', $row->pegawai->id).'">'.$row->pegawai->name.'</a>'.
                        '</p>'.
                        '<p class="my-0"><small>No. Pegawai /  NIP: '.$row->pegawai->no_pegawai.'</small></p>';
                    return $content;
                })
                ->addColumn('jumlah', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['jumlah'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('nominal', function($row){
                    return $row['jumlah'];
                })
                ->addColumn('keterangan', function($row){
                    $content = '<p class="my-0">'.$row->keterangan.'</p>';
                    return $content;
                })
                ->addColumn('action', function($row){
                    $btn = '<div class="d-flex justify-content-start">';
                    if($row->tipe == 'angsuran'){
                        $btn = $btn . '<form class="form-delete" action="'.route('potongan.instansi.delete', $row->id ).'" method="POST">'.
                        '<input type="hidden" name="_method" value="delete" />'.
                        '<input type="hidden" name="_token" id="csrf-token" value="'.csrf_token().'" />'.
                        '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                        '</form>';
                    } else {
                        $btn = $btn . '<form class="form-delete" action="'.route('potongan.internal.delete', $row->id ).'" method="POST">'.
                        '<input type="hidden" name="_method" value="delete" />'.
                        '<input type="hidden" name="_token" value="'.csrf_token().'" />'.
                        '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                        '</form>';
                    }
                    
                    return $btn;
                })
                ->rawColumns(['bulan', 'type', 'pegawai', 'keterangan', 'jumlah', 'action'])
                ->make(true);
        }
        return view('dashboard.instansi.list-potongan', [
            'potongans' => $potongans, 
            'jenispotongans' => $jenispotongans, 
            'pegawais' => $pegawais,
            'filterTahun' => $filterTahun,
            'filterBulan' => $filterBulan,
            'filterJenis' => $filterJenis,
            'filterPegawai' => $filterPegawai
        ]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $jenispotongans = [];
        $jenispotongans = JenisPotongan::all();
        if(auth()->user()->instansi && auth()->user()->menuroles == 'instansi'){
            $jenispotongans = auth()->user()->instansi->potongantypes;
        }
        $showData = false;
        $data = [];
        $count['valid'] = $count['invalid'] = 0;
        if($request->isMethod('post')) {
            if(!$request->bulan || !$request->tahun){
                return redirect()->back()->withInput()->with('error', 'Bulan dan tahun potongan harus diisi');;
            }
            if(!$request->potongan_type_id){
                return redirect()->back()->withInput()->with('error', 'Jenis potongan tidak boleh kosong');;
            }
            if(!$request->file('file')){
                return redirect()->back()->withInput()->with('error', 'Tidak ada file yang diupload');;
            }
            $rows = (new AngsuransImport)->toArray(request()->file('file'));
            
            foreach ($rows[0] as $key => $row) {
                if (isset($row)) {
                    $user = User::where('no_pegawai', str_replace(' ', '', $row[2]))->first();
                    $nip_exist = $user ? true : false;
                    $nama = $nip_exist ? $row[1] : 'NIP tidak ditemukan';
                    $count['valid'] = $nip_exist ? $count['valid'] + 1 : $count['valid'];
                    $count['invalid'] = !$nip_exist ? $count['invalid'] + 1 : $count['invalid'];
                    $data[$key] = [
                        'is_exist' => $nip_exist,
                        'pegawai_id' => $user ? $user->id : null,
                        'potongan_type_id' => $request['potongan_type_id'],
                        'bulan' => $request['bulan'],
                        'tahun' => $request['tahun'],
                        'nip' => $row[2],
                        'pegawai' => $nama,
                        'alamat' => $row[3],
                        'dasar_pemotongan' => $row[4],
                        'plafon' => $row[5],
                        'jangka_waktu' => $row[6],
                        'jatuh_tempo' => $row[7],//Carbon::createFromFormat('d-m-Y', str_replace("'", "", $row[7]))->format('y-m-d'),
                        'cicilan_ke' => $row[8],
                        'pokok' => $row[9],
                        'bunga' => $row[10],
                        'total' => $row[11],
                        'saldo_akhir' => $row[12],
                    ];
                }
            }
            $showData = true;
            session(['angsurans' => $data]);
        }

        return view('dashboard.instansi.input-potongan', compact('showData', 'data', 'request', 'count', 'jenispotongans'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = session('angsurans');
        try {
            foreach ($data as $key => $value) {
                Angsuran::where('pegawai_id', $value['pegawai_id'])
                        ->where('potongan_type_id', $value['potongan_type_id'])
                        ->where('bulan', $value['bulan'])
                        ->where('tahun', $value['tahun'])
                        ->delete();
                Angsuran::create([
                    'pegawai_id' => $value['pegawai_id'],
                    'potongan_type_id' => $value['potongan_type_id'],
                    'bulan' => $value['bulan'],
                    'tahun' => $value['tahun'],
                    'pegawai_nama' => $value['pegawai'],
                    'pegawai_kode' => $value['nip'],
                    'pegawai_alamat' => $value['alamat'],
                    'surat_dasar_pemotongan' => $value['dasar_pemotongan'],
                    'plafon' => $value['plafon'],
                    'jangka_waktu' => $value['jangka_waktu'],
                    'jatuh_tempo' => date('Y-m-d', strtotime($value['jatuh_tempo'])),
                    'angsuran_cicilan_ke' => $value['cicilan_ke'],
                    'angsuran_pokok' => $value['pokok'],
                    'angsuran_bunga' => $value['bunga'],
                    'angsuran_total' => $value['total'],
                    'saldo_akhir' => $value['saldo_akhir'],
                    'created_by' => auth()->user()->id
                ]);
            }

            return redirect()->route('potongan.instansi.index')->with('success', 'Potongan berhasil disimpan');
        } catch (\Throwable $th) {
            return back()->withError($th->getMessage())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $angsuran = Angsuran::find($id);
        if($angsuran){
            $angsuran->delete();
        }
        return redirect()->route('potongan.instansi.list')->with('success', 'Data potongan telah dihapus');
    }
}
