<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Instansi;
use App\Models\JenisPotongan;
use Illuminate\Support\Facades\DB;

class InstansiController extends Controller
{
    public function index()
    {
        $instansis = Instansi::all();
        return view('dashboard.instansi.index', compact('instansis'));
    }

    public function create()
    {
        $potongan_types = JenisPotongan::pluck('name', 'id');
        return view('dashboard.instansi.create', compact('potongan_types'));
    }

    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required',
            'address' => 'required',
            'phone' => 'required'
        ]);

        try {
            $instansi = Instansi::create([
                'name' => $request['name'],
                'address' => $request['address'],
                'phone' => $request['phone']
            ]);
            if($request['potongan_types']){
                $instansi->potongantypes()->attach($request['potongan_types']);
            }
            
            return redirect()->route('instansi.index')->with('success', 'Instansi '. $instansi->name. ' berhasil disimpan');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }

    public function edit($id)
    {
        $instansi = Instansi::find($id);
        $potongan_types = JenisPotongan::pluck('name', 'id');
        return view('dashboard.instansi.edit', compact('instansi', 'potongan_types'));
    }

    public function update(Request $request, $id)
    {
        $request->validate([
            'name' => 'required',
            'address' => 'required',
            'phone' => 'required'
        ]);

        $instansi = Instansi::find($id);
        try {
            $instansi->update([
                'name' => $request['name'],
                'address' => $request['address'],
                'phone' => $request['phone']
            ]);
            $instansi->potongantypes()->sync($request['potongan_types']);
            return redirect()->route('instansi.index')->with('success', 'Instansi '.$instansi->name.' berhasil diubah');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }

    public function destroy($id)
    {
        $instansi = Instansi::find($id);
        
        try {
            DB::beginTransaction();
            $instansi->potongantypes()->detach();
            $instansi->delete();
            DB::commit();
            return redirect()->route('instansi.index')->with('success', 'Instansi berhasil dihapus');
        } catch (\Throwable $th) {
            DB::rollback();
            return redirect()->back()->with('error', $th->getMessage());
        }
    }
}
