<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Spatie\DbDumper\Databases\MySql;
use App\Models\TransaksiInternal;
use App\Models\Instansi;
use Carbon\Carbon;
use PDF;
use Session;
use DataTables;

class KeuanganController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(Request $request)
    {
        $instansi = auth()->user()->instansi;
        $instansis = Instansi::all();
        if($instansi && auth()->user()->menuroles == 'instansi_internal'){
            $instansis = Instansi::where('id', $instansi->id)->get();
        }
        $filterTglAwal = $request['tgl_awal'];
        if(!$filterTglAwal) {
            $filterTglAwal = Carbon::now()->startOfMonth()->format('Y-m-d');
        }
        $filterTglAkhir = $request['tgl_akhir'];
        if(!$filterTglAkhir) {
            $filterTglAkhir = Carbon::now()->endOfMonth()->format('Y-m-d');
        }
        $filterInstansi = Session::get('transaksi_instansi', null);
        $filterInstansi = $request['instansi_id'];
        if($instansi && auth()->user()->menuroles == 'instansi_internal'){
            $filterInstansi = $instansi->id;
        }
        $transaksis = TransaksiInternal::with('instansi')->where('tanggal', '>=', $filterTglAwal)->where('tanggal', '<=', $filterTglAkhir);
        if($filterInstansi){
            $transaksis = $transaksis->where('instansi_id', $filterInstansi);
            Session::put('transaksi_instansi', $filterInstansi);
        }
        $transaksis = $transaksis->get();
        if($request->ajax()){
            return DataTables::of($transaksis)
                ->addIndexColumn()
                ->addColumn('tanggal', function($row){
                    return Carbon::createFromFormat('Y-m-d', $row['tanggal'])->format('d F Y');
                })
                ->addColumn('jenis', function($row){
                    switch($row['jenis']) {
                        case 'kas-masuk':
                            return '<span class="text-primary">Kas Masuk</span>';
                            break;
                        case 'kas-keluar':
                            return '<span class="text-danger">Kas Keluar</span>';
                            break;
                    }
                })
                ->addColumn('deskripsi', function($row){
                    $content = '<p class="my-0">'.$row['nama'].'</p><p class="my-0"><em>'.$row['keterangan'].'</em></p>';
                    return $content;
                })
                ->addColumn('nominal', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['jumlah'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('action', function($row){
                    $btn = '<div class="d-flex justify-content-start"><a href="'.route('keuangan.edit', $row->id ).'" class="btn btn-sm btn-warning mr-1"><span class="cil-pencil"></span> Edit</a>';
                    $btn = $btn . '<form class="form-delete" action="'.route('keuangan.delete', $row->id ).'" method="POST">'.
                    '<input type="hidden" name="_method" value="delete" />'.
                    '<input type="hidden" name="_token" id="csrf-token" value="'.csrf_token().'" />'.
                    '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                    '</form>';
                    return $btn;
                })
                ->rawColumns(['tanggal', 'jenis', 'deskripsi', 'nominal', 'action'])
                ->make(true);
        }
        return view('dashboard.keuangan.index', [
            'transaksis' => $transaksis, 
            'instansis' => $instansis,
            'filterTglAwal' => $filterTglAwal,
            'filterTglAkhir' => $filterTglAkhir,
            'filterInstansi' => $filterInstansi
        ]);
    }
    public function input()
    {
        $instansis = null;
        if(auth()->user()->instansi_id > 0) {
            $instansis = Instansi::where('id', auth()->user()->instansi_id)->get();
        } else {
            $instansis = Instansi::get();
        }
        return view('dashboard.keuangan.input', compact('instansis'));
    }
    
    public function store(Request $request) {
        $request->validate([
            'jenis' => 'required|in:kas-masuk,kas-keluar',
            'tanggal' => 'required',
            'instansi_id' => 'required',
            'nama' => 'required',
            'keterangan' => 'required',
            'jumlah' => 'required'
        ]);

        try {
            TransaksiInternal::create([
                'instansi_id' => $request['instansi_id'],//auth()->user()->instansi_id,
                'tanggal' => $request['tanggal'],
                'jenis' => $request['jenis'],
                'nama' => $request['nama'],
                'keterangan' => $request['keterangan'],
                'jumlah' => str_replace(".", "", $request['jumlah']),
            ]);

            return redirect('/keuangan/transaksi')->with('success', 'Transaksi berhasil disimpan');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    public function edit($id)
    {
        $transaksi = TransaksiInternal::with('instansi')->findOrFail($id);
        return view('dashboard.keuangan.edit', compact('transaksi'));
    }

    public function update(Request $request) {
        $request->validate([
            'jenis' => 'required|in:kas-masuk,kas-keluar',
            'tanggal' => 'required',
            'transaksi_id' => 'required',
            'nama' => 'required',
            'keterangan' => 'required',
            'jumlah' => 'required'
        ]);

        try {
            $transaksi = TransaksiInternal::findOrFail($request['transaksi_id']);
            $transaksi->tanggal = $request['tanggal'];
            $transaksi->jenis = $request['jenis'];
            $transaksi->nama = $request['jenis'];
            $transaksi->keterangan = $request['keterangan'];
            $transaksi->jumlah = str_replace(".", "", $request['jumlah']);
            $transaksi->save();
            return redirect('/keuangan/transaksi')->with('success', 'Transaksi berhasil diupdate');
        } catch (\Exception $e) {
            dd($e);
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    public function report()
    {
        $instansis = Instansi::get()->pluck('name', 'id');
        $instansi = auth()->user()->instansi;
        if($instansi && auth()->user()->menuroles == 'instansi_internal'){
            $instansis = Instansi::where('id', $instansi->id)->get()->pluck('name', 'id');
        }
        $saldo = 0;
        $datas = [];

        return view('dashboard.keuangan.report', compact('instansis', 'saldo', 'datas'));
    }

    public function generateReport(Request $request)
    {
        $instansis = Instansi::pluck('name', 'id');
        $nama_instansi = Instansi::where('id', $request['instansi_id'])->value('name');

        $transaksi_sebelum = TransaksiInternal::where('instansi_id', $request['instansi_id'])
            ->where('tanggal', '<', $request['date_from']);
        $kas_masuk = $transaksi_sebelum->where('jenis', 'kas-masuk')
            ->sum('jumlah');
        $kas_keluar = $transaksi_sebelum->where('jenis', 'kas-keluar')
            ->sum('jumlah');

        $saldo_awal = $kas_masuk - $kas_keluar;
        $saldo = number_format($saldo_awal, 0, '', '.');

        $transaksi = TransaksiInternal::where('instansi_id', $request['instansi_id'])
            ->whereDate('tanggal', '>=', $request['date_from'])
            ->whereDate('tanggal', '<=', $request['date_to'])
            ->select('tanggal', 'nama', 'jenis', 'jumlah')
            ->orderBy('tanggal', 'asc')
            ->get();

        $datas = [];
        foreach($transaksi as $key => $t) {
            $datas[$key]['tanggal'] = date("j F Y", strtotime($t['tanggal']));
            $datas[$key]['nama'] = $t['nama'];
            $datas[$key]['masuk'] = $t['jenis'] == 'kas-masuk' ? 'Rp '.number_format($t['jumlah'], 0, '', '.') : '-';
            $datas[$key]['keluar'] = $t['jenis'] == 'kas-keluar' ? 'Rp '.number_format($t['jumlah'], 0, '', '.') : '-';
            
            if($t['jenis'] == 'kas-masuk') {
                $hitung = $saldo_awal + $t['jumlah'];
            } else if($t['jenis'] == 'kas-keluar') {
                $hitung = $saldo_awal - $t['jumlah'];
            }

            $datas[$key]['saldo'] = 'Rp '.number_format($hitung, 0, '', '.');
            $saldo_awal = $hitung;
        }

        session([
            'saldo_awal' => $saldo,
            'instansi' => $nama_instansi,
            'date_from' => date("j F Y", strtotime($request['date_from'])),
            'date_to' => date("j F Y", strtotime($request['date_to'])),
            'laporan_transaksi' => $datas
        ]);
        $request->flash();
        return view('dashboard.keuangan.report', compact('instansis', 'saldo', 'datas'));
    }

    public function print()
    {
        $saldo = session('saldo_awal');
        $instansi = session('instansi');
        $date_from = session('date_from');
        $date_to = session('date_to');
        $datas = session('laporan_transaksi');

        $pdf = PDF::setPaper('legal', 'portrait')
                    ->loadView('dashboard.keuangan.print', [
                        'saldo' => $saldo,
                        'instansi' => $instansi,
                        'date_from' => $date_from,
                        'date_to' => $date_to,
                        'datas' => $datas,
                    ]);
        return $pdf->stream('Laporan Keuangan.pdf');
    }

    public function destroy($id)
    {
        $transaksi = TransaksiInternal::find($id);
        if($transaksi){
            $transaksi->delete();
        }
        return redirect()->route('keuangan.index')->with('success', 'Data transaksi telah dihapus');
    }
}
