<?php

namespace App\Http\Controllers;

use App\Models\JenisPotongan;
use App\Models\JenisPenambahan;
use App\Models\SalarySlip;
use App\Models\User;
use App\Models\Angsuran;
use App\Models\PotonganInternal;
use App\Models\PenambahanInternal;
use App\Models\SalaryPenambahan;
use Illuminate\Http\Request;
use DataTables;
use Session;
use Illuminate\Support\Facades\DB;
use PDF;
use App\Exports\RekapExport;
use Maatwebsite\Excel\Facades\Excel;

class PenerimaanController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if(auth()->user()->menuroles == 'pegawai'){
            $pegawais = User::where('id', auth()->user()->id)->get();
        } else {
            $pegawais = User::where('menuroles', 'pegawai')->get();
        }
        if(auth()->user()->menuroles == 'pegawai'){
            $slips = SalarySlip::with(['riwayat.pegawai', 'potongans', 'penambahans'])->where('user_id', auth()->user()->id);
        } else {
            $slips = SalarySlip::with(['riwayat.pegawai', 'potongans', 'penambahans']);
        }
        $filterTahun = Session::get('penerimaan_filter_tahun', null);
        if(($request['tahun'] != null) && ($filterTahun != $request['tahun'])){
            $filterTahun = $request['tahun'];
        }
        $filterBulan = Session::get('penerimaan_filter_bulan', null);
        if(($request['bulan'] != null) && ($filterBulan != $request['bulan'])){
            $filterBulan = $request['bulan'];
        }
        $filterJenis = Session::get('penerimaan_filter_jenis', null);
        if(($request['jenis'] != null) && ($filterJenis != $request['jenis'])){
            $filterJenis = $request['jenis'];
        }
        $filterJenisPegawai = Session::get('penerimaan_filter_jenispegawai');
        // dd($filterJenisPegawai);
        if($filterJenisPegawai != $request['jenispegawai']){
            $filterJenisPegawai = $request['jenispegawai'];
            
        }
        // dd(Session::get('penerimaan_filter_jenispegawai'), $request['jenispegawai']);
        $filterPegawai = Session::get('penerimaan_filter_pegawai', null);
        if($filterPegawai != $request['pegawai']){
            $filterPegawai = $request['pegawai'];
        }
        if($filterTahun){
            $slips = $slips->where('year', $filterTahun);
            Session::put('penerimaan_filter_tahun', $filterTahun); 
        }
        if($filterBulan){
            $slips = $slips->where('month', $filterBulan);
            Session::put('penerimaan_filter_bulan', $filterBulan); 
        }
        if($filterJenis){
            $slips = $slips->where('type', $filterJenis);
            Session::put('penerimaan_filter_jenis', $filterJenis); 
        }
        if($filterJenisPegawai){
            //get pegawai berdasarkan jenis
            $ids = User::where('pegawai_type', $filterJenisPegawai)->pluck('id')->toArray();
            $slips = $slips->whereIn('user_id', $ids);
        }
        if($filterPegawai){
            $slips = $slips->where('user_id', $filterPegawai);
            Session::put('penerimaan_filter_pegawai', $filterPegawai); 
        }
        $slips = $slips->orderBy('type')->orderBy('month', 'asc')->get()->sortBy('riwayat.pegawai.pegawai_type');
        // dd($slips);
        $slips = $slips->map(function ($slip, $key) {
            $jenis = config('constants.jenis_riwayat.' . $slip->type);
            $slip['tipe_penerimaaan'] = config('constants.salary_types.' . $slip->type);
            $totalPembayaran = $slip->riwayat->$jenis;
            $potonganBRI = 0;
            $potonganNonBRI = 0;
            $totalPotongan = 0;
            foreach($slip->penambahans as $penambahan) {
                $totalPembayaran += $penambahan->amount;
            }
            foreach($slip->potongans as $potongan) {
                $totalPotongan += $potongan->amount;
            }
            $totalPenerimaan = $totalPembayaran - $totalPotongan;
            $slip['total_pembayaran'] = $totalPembayaran;
            $slip['total_potongan'] = $totalPotongan;
            $slip['total_penerimaan'] = $totalPenerimaan;
            return $slip;
        });
        // $penambahans = SalaryPenambahan::
        session([
            'slips' => $slips->groupBy('type'),
            'year' => $request['tahun'],
            'month' => $request['bulan'],
        ]);
        if($request->ajax()){
            Session::put('penerimaan_filter_jenispegawai', $filterJenisPegawai);
            return DataTables::of($slips)
                ->addIndexColumn()
                ->addColumn('bulan', function($row){
                    return config('constants.bulans.' . $row->month);
                })
                ->addColumn('type', function($row){
                    return $row['tipe_penerimaaan'];
                })
                ->addColumn('pegawai', function($row){
                    $content = '<p class="my-0">'.
                        '<a href="'.route('pegawai.show', $row->riwayat->pegawai->id).'">'.$row->riwayat->pegawai->name.'</a>'.
                        '</p>'.
                        '<p class="my-0"><small>No. Pegawai /  NIP: '.$row->riwayat->pegawai->no_pegawai.'</small></p>';
                    return $content;
                })
                ->addColumn('jumlah', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['total_pembayaran'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('potongan', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['total_potongan'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('potonganBri', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row->potongans()->where('id', 20)->sum('amount'),0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('potonganNonBri', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row->potongans()->where('id', '!=', 20)->sum('amount'),0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('diterima', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['total_penerimaan'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('keterangan', function($row){
                    switch ($row->approve_pegawai) {
                        case 1:
                            return "<span class='text-success'><i class='cil-check-circle'></i> Disetujui Pegawai</span>";
                            break;
                        
                        case -1:
                            return "<span class='text-danger'><i class='cil-comment-bubble'></i> Keberatan dengan keterangan : ". $row->remark."</span>";
                            break;
                            
                        default:
                            return '-';
                            break;
                    }
                    return config('constants.salary_types.' . $row->type);
                })
                ->addColumn('action', function($row){
                    $content = '<a href="'.route('penerimaan.detail', $row->id).'" class="btn btn-sm btn-info"><span class="cil-spreadsheet mr-1"></span> Detail</a>';
                    if(auth()->user()->menuroles == 'admin' || auth()->user()->menuroles == 'kasubag_keuangan' || auth()->user()->menuroles == 'petugas_keuangan'){
                        $content .= '<a href="'.route('penerimaan.edit-potongan', $row->id).'" class="btn btn-sm btn-warning mt-1"><span class="cil-pencil mr-1"></span> Edit Komponen</a>';
                        $content = $content . '<form class="form-delete" action="'.route('penerimaan.delete-slip', $row->id ).'" method="POST">'.
                        '<input type="hidden" name="_method" value="delete" />'.
                        '<input type="hidden" name="_token" id="csrf-token-'.$row->id.'" value="'.csrf_token().'" />'.
                        '<button class="btn btn-sm btn-danger mt-1"><span class="cil-trash mr-1"></span> Hapus</button>'.
                        '</form>';
                    }
                    
                    return $content;
                })
                ->rawColumns(['bulan', 'type', 'pegawai', 'jumlah', 'potongan', 'potonganBri', 'potonganNonBri', 'diterima', 'keterangan', 'action'])
                ->make(true);
        }
        return view('dashboard.penerimaan.index', [
            'slips' => $slips,
            'pegawais' => $pegawais,
            'filterTahun' => $filterTahun,
            'filterBulan' => $filterBulan,
            'filterJenis' => $filterJenis,
            'filterJenisPegawai' => $filterJenisPegawai,
            'filterPegawai' => $filterPegawai
        ]);
    }

    public function download()
    {
        $slips = session('slips');
        $year = session('year');
        $month = session('month');

        $pdf = PDF::setPaper('legal', 'landscape')
                    ->loadView('dashboard.penerimaan.print-list', [
                        'slips' => $slips,
                        'year' => $year,
                        'month' => $month,
                    ]);
        return $pdf->stream('Rekapitulasi-Penerimaan-'.$year.'-'.$month.'.pdf');
    }

    public function downloadExcel()
    {
        $slips = session('slips');
        $year = session('year');
        $month = session('month');
        return Excel::download(new RekapExport($slips, $year, $month), 'RekapExport.xlsx');
    }

    public function downloadDaftar()
    {
        $slips = session('slips');
        $year = session('year');
        $month = session('month');

        $pdf = PDF::setPaper('legal', 'landscape')
                    ->loadView('dashboard.penerimaan.print-list2', [
                        'slips' => $slips,
                        'year' => $year,
                        'month' => $month,
                    ]);
        return $pdf->stream('Daftar-Penerimaan-'.$year.'-'.$month.'.pdf');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function generate(Request $request)
    {
        if($request->ajax()){
            $col = config('constants.jenis_riwayat.' . $request['jenis']);
            $data = User::pegawai()->has('riwayat')->with([
                'riwayat' => function($q) {
                    $q->where('status', 1);
                },
                'riwayat.jabatan',
                'riwayat.pangkat'
            ])
            ->whereHas('riwayat', function ($query) use($col) {
                $query->where($col, '>', 0);
            })
            ->where('status', 1)
            ->orderBy('created_at', 'desc')
            ->get()->toArray();
    
            $user_ids = SalarySlip::where('type', $request['jenis'])
                ->where('year', $request['tahun'])
                ->where('month', $request['bulan'])
                ->pluck('user_id')
                ->toArray();
            $array = [];
            foreach ($data as $key => $value) {
                if(!in_array($value['id'], $user_ids)){
                    $array[$key] = 
                    [
                        'user_id' => $value['id'],
                        'riwayat_pegawai_id' => $value['riwayat'][0]['id'],
                        'no' => '12345678',
                        'type' => $request['jenis'],
                        'year' => $request['tahun'],
                        'month' => $request['bulan'],
                        'document_date' => date("Y-m-d")
                    ];
                }
            }
            session([
                'slips' => $array
            ]);

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('jabatan', function($row){
                    return $row['riwayat'][0]['jabatan']['name'];
                })
                ->addColumn('pangkat', function($row){
                    return $row['riwayat'][0]['pangkat']['name'];
                })
                ->addColumn('jumlah', function($row)use($request){
                    return number_format($row['riwayat'][0][config('constants.jenis_riwayat.' . $request['jenis'])],0,',','.');
                })
                ->rawColumns(['jabatan', 'pangkat', 'jumlah'])
                ->make(true);
        }
        return view('dashboard.penerimaan.generate');
    }

    public function editPotongan($id)
    {
        $slip = SalarySlip::where('id', $id)->with(['riwayat', 'potongans', 'penambahans'])->first();
        $slip = $slip->setRelation('potongans', $slip->potongans->keyBy('potongan_type_id'));
        $slip = $slip->setRelation('penambahans', $slip->penambahans->keyBy('penambahan_type_id'));
        $jenisPotongans = JenisPotongan::with('instansi')->get();
        $jenisPenambahans = JenisPenambahan::get();
        $existingPotongan = [];
        foreach($jenisPotongans as $key=>$item) {
            if($item->instansi()->first()){
                $dataAngsuran = Angsuran::where('potongan_type_id', $item->id)
                                        ->where('pegawai_id', $slip->user_id)
                                        ->where('bulan', $slip->month)
                                        ->where('tahun', $slip->year)
                                        ->first();
                $dataPotonganInternal = PotonganInternal::where('potongan_type_id', $item->id)
                                        ->where('pegawai_id', $slip->user_id)
                                        ->where('bulan', $slip->month)
                                        ->where('tahun', $slip->year)
                                        ->first();
                if($dataAngsuran){
                    $existingPotongan[$item->id]['instansi'] = $item->instansi()->first()->name;
                    $existingPotongan[$item->id]['tipe'] = 'angsuran';
                    $existingPotongan[$item->id]['surat_dasar_pemotongan'] = $dataAngsuran['surat_dasar_pemotongan'];
                    $existingPotongan[$item->id]['plafon'] = $dataAngsuran['plafon'];
                    $existingPotongan[$item->id]['saldo_akhir'] = $dataAngsuran['saldo_akhir'];
                    $existingPotongan[$item->id]['angsuran_cicilan_ke'] = $dataAngsuran['angsuran_cicilan_ke'];
                    $existingPotongan[$item->id]['angsuran_pokok'] = $dataAngsuran['angsuran_pokok'];
                    $existingPotongan[$item->id]['angsuran_bunga'] = $dataAngsuran['angsuran_bunga'];
                    $potonganTotal = $dataAngsuran['angsuran_total'];
                    //get potongan yang sudah dihitung pada penerimaan lain, jika sudah, tambahkan nilai dibayar
                    $potongansPaid = SalarySlip::join('salary_potongans', 'salary_potongans.salary_slip_id', '=', 'salary_slips.id')
                                                    ->where('user_id', $slip->user_id)
                                                    ->where('month', $slip->month)
                                                    ->where('year', $slip->year)
                                                    ->where('salary_potongans.potongan_type_id', $item->id)
                                                    ->sum('salary_potongans.amount');
                    $existingPotongan[$item->id]['total'] = $potonganTotal;
                    $existingPotongan[$item->id]['dibayar'] = $potongansPaid;
                    $existingPotongan[$item->id]['sisa'] = $potonganTotal-$potongansPaid;
                } else if($dataPotonganInternal) {
                    $existingPotongan[$item->id]['instansi'] = $item->instansi()->first()->name;
                    $existingPotongan[$item->id]['tipe'] = 'potongan';
                    $potonganTotal = $dataPotonganInternal['nominal'];
                    //get potongan yang sudah dihitung pada penerimaan lain, jika sudah, tambahkan nilai dibayar
                    $potongansPaid = SalarySlip::join('salary_potongans', 'salary_potongans.salary_slip_id', '=', 'salary_slips.id')
                                                    ->where('user_id', $slip->user_id)
                                                    ->where('month', $slip->month)
                                                    ->where('year', $slip->year)
                                                    ->where('salary_potongans.potongan_type_id', $item->id)
                                                    ->sum('salary_potongans.amount');
                    $existingPotongan[$item->id]['total'] = $potonganTotal;
                    $existingPotongan[$item->id]['dibayar'] = $potongansPaid;
                    $existingPotongan[$item->id]['sisa'] = $potonganTotal-$potongansPaid;
                }
            }
        }
        $existingPenambahan = [];
        foreach($jenisPenambahans as $key=>$item) {
            $dataPenambahanInternal = PenambahanInternal::where('penambahan_type_id', $item->id)
                                        ->where('pegawai_id', $slip->user_id)
                                        ->where('bulan', $slip->month)
                                        ->where('tahun', $slip->year)
                                        ->first();
            if($dataPenambahanInternal) {
                $existingPenambahan[$item->id]['instansi'] = env('APP_COMPANY');
                $existingPenambahan[$item->id]['tipe'] = 'penambahan';
                $penambahanTotal = $dataPenambahanInternal['nominal'];
                //get potongan yang sudah dihitung pada penerimaan lain, jika sudah, tambahkan nilai dibayar
                $penambahanPaid = SalarySlip::join('salary_penambahans', 'salary_penambahans.salary_slip_id', '=', 'salary_slips.id')
                                                ->where('user_id', $slip->user_id)
                                                ->where('month', $slip->month)
                                                ->where('year', $slip->year)
                                                ->where('salary_penambahans.penambahan_type_id', $item->id)
                                                ->sum('salary_penambahans.amount');
                $existingPenambahan[$item->id]['total'] = $penambahanTotal;
                $existingPenambahan[$item->id]['dibayar'] = $penambahanPaid;
                $existingPenambahan[$item->id]['sisa'] = $penambahanTotal-$penambahanPaid;
            }
        }
        return view('dashboard.penerimaan.edit-potongan',
            compact('jenisPotongans', 'jenisPenambahans', 'slip', 'existingPotongan', 'existingPenambahan')
        );
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = session('slips');
        DB::beginTransaction();

        try {
            DB::table('salary_slips')->insert($data);
            DB::commit();
            return response()->json(['success' => true]);
        } catch (\Throwable $e) {
            DB::rollback();
            throw $e;
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Pangkat  $pangkat
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $jenisPotongans = JenisPotongan::all();
        $jenisPenambahans = JenisPenambahan::all();
        $slip = SalarySlip::with([
            'pegawai',
            'riwayat' => function($q){
                $q->where('status', 1);
            },
            'potongans.jenis',
            'penambahans.jenis'
        ])->where('id', $id)->first();
        $slip = $slip->setRelation('potongans', $slip->potongans->keyBy('potongan_type_id'));
        $slip = $slip->setRelation('penambahans', $slip->penambahans->keyBy('penambahan_type_id'));

        return view('dashboard.penerimaan.show', [
            'jenisPotongans' => $jenisPotongans,
            'jenisPenambahans' => $jenisPenambahans,
            'slip' => $slip,
            'id' => $id
        ]);
    }

    public function print($id)
    {
        $jenisPotongans = JenisPotongan::all();
        $jenisPenambahans = JenisPenambahan::all();
        $slip = SalarySlip::with([
            'pegawai',
            'riwayat' => function($q){
                $q->where('status', 1);
            },
            'potongans.jenis',
            'penambahans.jenis'
        ])->where('id', $id)->first();
        $slip = $slip->setRelation('potongans', $slip->potongans->keyBy('potongan_type_id'));
        $slip = $slip->setRelation('penambahans', $slip->penambahans->keyBy('penambahan_type_id'));

        $pdf = PDF::setPaper('legal', 'portrait')
                    ->loadView('dashboard.penerimaan.print', [
                        'jenisPotongans' => $jenisPotongans,
                        'jenisPenambahans' => $jenisPenambahans,
                        'slip' => $slip
                    ]);
        return $pdf->stream('Slip Gaji.pdf');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Pangkat  $pangkat
     * @return \Illuminate\Http\Response
     */
    public function edit(Pangkat $pangkat)
    {
        return view('dashboard.pangkat.edit', ['pangkat' => $pangkat]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Pangkat  $pangkat
     * @return \Illuminate\Http\Response
     */
    public function updatePotongan(Request $request, $id)
    {
        $slip = SalarySlip::find($id);
        foreach ($request['potongan'] as $potongan_type_id => $amount) {
            if($slip->potongans->contains('potongan_type_id', $potongan_type_id)){
                $slip->potongans()->where('potongan_type_id', $potongan_type_id)->update([
                    'desc' => !empty($request->desc[$potongan_type_id])?$request->desc[$potongan_type_id]:'',
                    'amount' => str_replace(".", "", $amount)
                ]);
            } else {
                if($amount != 0 && $amount != null){
                    $slip->potongans()->create([
                        'potongan_type_id' => $potongan_type_id,
                        'desc' => !empty($request->desc[$potongan_type_id])?$request->desc[$potongan_type_id]:'',
                        'amount' => str_replace(".", "", $amount)
                    ]);
                }
            }
        }
        foreach ($request['penambahan'] as $penambahan_type_id => $amount) {
            if($slip->penambahans->contains('penambahan_type_id', $penambahan_type_id)){
                $slip->penambahans()->where('penambahan_type_id', $penambahan_type_id)->update([
                    'desc' => !empty($request->desc[$penambahan_type_id])?$request->desc[$penambahan_type_id]:'',
                    'amount' => str_replace(".", "", $amount)
                ]);
            } else {
                if($amount != 0 && $amount != null){
                    $slip->penambahans()->create([
                        'penambahan_type_id' => $penambahan_type_id,
                        'desc' => !empty($request->desc[$penambahan_type_id])?$request->desc[$penambahan_type_id]:'',
                        'amount' => str_replace(".", "", $amount)
                    ]);
                }
            }
        }
        if($request->submit_and_back && $request->submit_and_back == 1) {
            return redirect()->route('penerimaan.index')->with('success', 'Potongan berhasil diupdate');
        } else {
            return redirect()->route('penerimaan.edit-potongan', $id)->with('success', 'Potongan berhasil diupdate');
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Pangkat  $pangkat
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $slip = SalarySlip::find($id);
            $slip->delete();
            return redirect()->route('penerimaan.index')->with('success', 'Data penerimaan telah dihapus');
        } catch (\Throwable $th) {
            return redirect()->route('penerimaan.index')->with('error', $th->getMessage());
        }
    }

    public function agreeCheck(Request $request) {
        if($request->ajax()) {
            // check if has signature
            $hasSign = User::where('id', $request['pegawai_id'])->value('signature');
            if($hasSign !== null) {
                return response()->json([
                    "message" => "Tanda tangan tervalidasi"
                ]);
            }
            else {
                return response()->json([
                    "message" => "Anda belum menambahkan tanda tangan. Tambah tanda tangan di halaman profil"
                ], 401);
            }

        }
    }

    public function agree(Request $request) {
        if($request->ajax()) {
            try {
                $slip = SalarySlip::where('id', $request['id'])->update([
                    'remark' => null,
                    'approve_pegawai' => 1
                ]);

                return response()->json([
                    "message" => "Anda berhasil menyetujui penerimaan ini."
                ], 200);
            } catch (\Exception $e) {
                return response()->json([
                    "message" => $e->getMessage()
                ]);
            }
        }
    }

    public function keberatan(Request $request, $id) {
        $request->validate([
            'remark' => 'required'
        ]);

        try {
            $slip = SalarySlip::where('id', $id)->update([
                'remark' => $request['remark'],
                'approve_pegawai' => -1
            ]);
            
            return redirect()->back()->with('success', 'Keberatan berhasil dikirim');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }
}
