<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use DataTables;

class PermissionsController extends Controller
{
    public function index(Request $request)
    {
        if($request->ajax()){
            $permissions = Permission::select('id', 'name')->get();
            return DataTables::of($permissions)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    return '<a href="'.route('permissions.show', $row->id ).'" class="btn btn-sm btn-info mr-2"><span class="cil-user mr-1"></span> Detail</a>'.
                    '<a href="'.route('permissions.edit', $row->id ).'" class="btn btn-sm btn-success mr-2"><span class="cil-pencil mr-1"></span> Edit</a>';
                 })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('dashboard.permissions.index');
    }

    public function create()
    {
        return view('dashboard.permissions.create');
    }

    public function store(Request $request)
    {
        $permission = Permission::create(['name' => $request['name']]);
        $request->session()->flash('message', 'Successfully created permission');
        return redirect()->route('permissions.create');
    }

    public function edit($id)
    {
        $permission = Permission::where('id', $id)->first();
        return view('dashboard.permissions.edit', [
            'permission' => $permission
        ]);
    }

    public function update(Request $request, $id)
    {
        $request->validate([
            'name' => 'required'
        ]);

        $permission = Permission::find($id);
        $permission->name = $request['name'];
        $permission->save();

        $request->session()->flash('message', 'Successfully updated permission');
        return redirect()->route('permissions.edit', $id);
    }

    public function show($id)
    {
        $permission = Permission::where('id', $id)->with('roles')->first();
        return view('dashboard.permissions.show', [
            'permission' => $permission
        ]);
    }
}
