<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Imports\PotonganInternalImport;
use App\Models\User;
use App\Models\PotonganInternal;
use App\Models\JenisPotongan;
use Carbon\Carbon;
use DataTables;
use Session;
use Illuminate\Support\Facades\DB;
use App\Exports\PegawaiExport;
use Maatwebsite\Excel\Facades\Excel;

class PotonganInternalController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function list(Request $request)
    {
        $instansi = auth()->user()->instansi;
        $jenispotongans = JenisPotongan::all();
        if($instansi && auth()->user()->menuroles == 'instansi_internal'){
            $jenispotongans = $instansi->potongantypes;
        }
        if(auth()->user()->menuroles == 'pegawai'){
            $pegawais = User::where('id', auth()->user()->id)->get();
        } else {
            $pegawais = User::where('menuroles', 'pegawai')->get();
        }
        $filterTahun = Session::get('potongan_internal_filter_tahun', null);
        if(!$filterTahun && $request['tahun']){
            $filterTahun = $request['tahun'];
        }
        $filterBulan = Session::get('potongan_internal_filter_bulan', null);
        if(!$filterBulan && $request['bulan']){
            $filterBulan = $request['bulan'];
        }
        $filterJenis = Session::get('potongan_internal_filter_jenis', null);
        if(!$filterJenis != $request['jenis']){
            $filterJenis = $request['jenis'];
        }
        $filterPegawai = Session::get('potongan_internal_filter_pegawai', null);
        if(!$filterPegawai != $request['pegawai']){
            $filterPegawai = $request['pegawai']; 
        }
        $potongans = PotonganInternal::with(['pegawai', 'jenis']);
        if($filterTahun){
            $potongans = $potongans->where('tahun', $filterTahun);
            Session::put('potongan_internal_filter_tahun', $filterTahun); 
        }
        if($filterBulan){
            $potongans = $potongans->where('bulan', $filterBulan);
            Session::put('potongan_internal_filter_bulan', $filterBulan);
        }
        if($filterJenis){
            $potongans = $potongans->where('potongan_internal_type_id', $filterJenis);
            Session::put('potongan_internal_filter_jenis', $filterJenis);
        }
        if($filterPegawai){
            $potongans = $potongans->where('pegawai_id', $filterPegawai);
            Session::put('potongan_internal_filter_pegawai', $filterPegawai);
        }
        $potongans = $potongans->get();
        if($request->ajax()){
            return DataTables::of($potongans)
                ->addIndexColumn()
                ->addColumn('bulan', function($row){
                    return config('constants.bulans.' . $row->bulan);
                })
                ->addColumn('type', function($row){
                    return $row->jenis->name;
                })
                ->addColumn('pegawai', function($row){
                    $content = '<p class="my-0">'.
                        '<a href="'.route('pegawai.show', $row->pegawai->id).'">'.$row->pegawai->name.'</a>'.
                        '</p>'.
                        '<p class="my-0"><small>No. Pegawai /  NIP: '.$row->pegawai->no_pegawai.'</small></p>';
                    return $content;
                })
                ->addColumn('nominal', function($row){
                    $content = '<div class="d-flex justify-content-between align-items-center">'.
                        '<p class="my-0">Rp</p>'.
                        '<p class="my-0">'.
                        number_format($row['angsuran_total'],0,',','.')
                        .'</p>'.
                    '</div>';
                    return $content;
                })
                ->addColumn('action', function($row){
                    $btn = '<div class="d-flex justify-content-start">';
                    $btn = $btn . '<form class="form-delete" action="'.route('potongan.internal.delete', $row->id ).'" method="POST">'.
                    '<input type="hidden" name="_method" value="delete" />'.
                    '<input type="hidden" name="_token" value="'.csrf_token().'" />'.
                    '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                    '</form>';
                    return $btn;
                })
                ->rawColumns(['bulan', 'type', 'pegawai', 'keterangan', 'jumlah', 'action'])
                ->make(true);
        }
        return view('dashboard.instansi.list-potongan-internal', [
            'potongans' => $potongans, 
            'jenispotongans' => $jenispotongans, 
            'pegawais' => $pegawais,
            'filterTahun' => $filterTahun,
            'filterBulan' => $filterBulan,
            'filterJenis' => $filterJenis,
            'filterPegawai' => $filterPegawai
        ]);
    }

    public function export() 
    {
        return Excel::download(new PegawaiExport, 'Daftar-Potongan.xlsx');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $jenispotongans = JenisPotongan::all();
        if(auth()->user()->instansi && auth()->user()->menuroles == 'instansi_internal'){
            $jenispotongans = auth()->user()->instansi->potongantypes;
        }
        $showData = false;
        $data = [];
        $count['valid'] = $count['invalid'] = 0;
        if($request->isMethod('post')) {
            if(!$request->bulan || !$request->tahun){
                return redirect()->back()->withInput()->with('error', 'Bulan dan tahun potongan harus diisi');;
            }
            if(!$request->potongan_type_id){
                return redirect()->back()->withInput()->with('error', 'Jenis potongan tidak boleh kosong');;
            }
            if(!$request->file('file')){
                return redirect()->back()->withInput()->with('error', 'Tidak ada file yang diupload');;
            }
            $rows = (new PotonganInternalImport)->toArray(request()->file('file'));
            foreach ($rows[0] as $key => $row) {
                if (isset($row)) {
                    $user = User::where('no_pegawai', trim($row[1],'\'"'))->first();
                    $nip_exist = $user ? true : false;
                    $nama = $nip_exist ? $row[2] : 'NIP tidak ditemukan';
                    $count['valid'] = $nip_exist ? $count['valid'] + 1 : $count['valid'];
                    $count['invalid'] = !$nip_exist ? $count['invalid'] + 1 : $count['invalid'];
                    $data[$key] = [
                        'is_exist' => $nip_exist,
                        'pegawai_id' => $user ? $user->id : null,
                        'potongan_type_id' => $request['potongan_type_id'],
                        'bulan' => $request['bulan'],
                        'tahun' => $request['tahun'],
                        'nip' => trim($row[1],'\'"'),
                        'pegawai' => $nama,
                        'total' => $row[4],
                    ];
                }
            }
            $showData = true;
            session(['potongans' => $data]);
        }

        return view('dashboard.instansi.input-potongan-internal', compact('showData', 'data', 'request', 'count', 'jenispotongans'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = session('potongans');
        try {
            foreach ($data as $key => $value) {
                //delete existing potongan 
                PotonganInternal::where('potongan_type_id', $value['potongan_type_id'])
                                ->where('bulan', $value['bulan'])
                                ->where('tahun', $value['tahun'])
                                ->where('pegawai_id', $value['pegawai_id'])
                                ->delete();
                PotonganInternal::updateOrCreate([
                    'pegawai_id' => $value['pegawai_id'],
                    'potongan_type_id' => $value['potongan_type_id'],
                    'bulan' => $value['bulan'],
                    'tahun' => $value['tahun']
                ],[
                    'nominal' => $value['total'],
                    'created_by' => auth()->user()->id
                ]);
            }

            return redirect()->route('potongan.internal.index')->with('success', 'Potongan Internal berhasil disimpan');
        } catch (\Throwable $th) {
            return back()->withError($th->getMessage())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $potongan = PotonganInternal::find($id);
        if($potongan){
            $potongan->delete();
        }
        return redirect()->route('potongan.instansi.list')->with('success', 'Data potongan telah dihapus');
    }
}
