<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Agama;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;

class ProfilController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function show()
    {
        $user = auth()->user();
        
        // signature
        $signatureUri = null;
        if ($user->signature !== null && file_exists('uploads/'.$user->signature)){
            $path = 'uploads/'.$user->signature;
            $type = pathinfo($path, PATHINFO_EXTENSION);
            $data = file_get_contents($path);
            $signatureUri = 'data:image/' . $type . ';base64,' . base64_encode($data);
        }

        $religions = Agama::pluck('name', 'id');
        return view('dashboard.profil.show', compact('user', 'religions', 'signatureUri'));
    }

    public function update(Request $request, $id)
    {
        $request->validate([
            'name' => 'required',
            'email' => 'required',
            'phone' => 'required'
        ]);

        try {
            $user = User::find($id);
            $user->name = $request['name'];
            $user->email = $request['email'];
            if($request['password'] != null){
                $user->password = Hash::make($request['password']);
            }
            $user->gender = $request['gender'] ?: null;
            $user->birth_place = $request['birth_place'] ?: null;
            $user->birth_date = $request['birth_date'] ?: null;
            $user->phone = $request['phone'];
            $user->address = $request['address'] ?: null;
            $user->religion_id = $request['religion_id'] ?: null;
            if ($request->file('photo')) {
                $photo = $request->file('photo')->store('pegawai', ['disk' => 'public']);
                $user->photo = $photo;
            }
            if ($request->file('ttdimage')) {
                $ttd = $request->file('ttdimage')->store('signature', ['disk' => 'public']);
                $user->signature = $ttd;
            }
            // if ($request['signature']){
            //     $data_uri = $request["signature"];
            //     $encoded_image = explode(",", $data_uri)[1];
            //     $decoded_image = base64_decode($encoded_image);
            //     $name = "signature/sign-".time().".jpg";
            //     $signature = Storage::put($name, $decoded_image);
            //     $user->signature = $name;
            // }
            $user->save();
            return redirect()->back()->with('success', 'Profil berhasil diperbarui');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage());
        }
    }
}
