<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Promo;
use Storage;

class PromoController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        if(auth()->user()->menuroles == 'admin'){
            $myPromos = Promo::all();
        } if(auth()->user()->menuroles == 'instansi'){
            $myPromos = Promo::where('instansi_id', auth()->user()->instansi_id)->get();
        }
        return view('dashboard.promo.index', compact('myPromos'));
    }

    public function create()
    {
        return view('dashboard.promo.create');
    }

    public function store(Request $request)
    {
        $request->validate([
            'image' => 'required',
            'description' => 'required',
            'active' => 'required'
        ]);

        try {
            $promo = Promo::create([
                'instansi_id' => auth()->user()->instansi_id,
                'image' => $request->file('image')->store('promo', ['disk' => 'public']),
                'description' => $request['description'],
                'active' => $request['active'],
                'kontak' => $request['kontak'],
                'link' => $request['link'],
            ]);
            return redirect()->route('promo.index')->with('success', 'Promo berhasil disimpan');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }

    public function edit($id)
    {
        $promo = Promo::find($id);
        return view('dashboard.promo.edit', compact('promo'));
    }
    
    public function update($id, Request $request)
    {
        $request->validate([
            'description' => 'required',
            'active' => 'required'
        ]);

        try {
            $promo = Promo::find($id);
            $promo->update([
                'image' => $request->file('image')->store('promo', ['disk' => 'public']),
                'description' => $request['description'],
                'active' => $request['active'],
                'kontak' => $request['kontak'],
                'link' => $request['link'],
            ]);
            return redirect()->route('promo.index')->with('success', 'Promo berhasil diubah');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        } 
    }

    public function destroy($id)
    {
        $promo = Promo::find($id);
        if($promo){
            $promo->delete();
        }
        return redirect()->route('promo.index')->with('success', 'Data promo telah dihapus');
    }
}
