<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Permission\Traits\HasRoles;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class User extends Authenticatable
{
    use Notifiable;
    use SoftDeletes;
    use HasRoles;
    use HasFactory;
    
    protected $table = 'users';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 
        'email', 
        'username',
        'password',
        'menuroles', 
        'no_pegawai', 
        'pegawai_type',
        'gender', 
        'birth_place', 
        'birth_date',
        'phone',
        'address',
        'religion_id',
        'photo',
        'account_bank',
        'instansi_id',
        'account_no',
        'npwp',
        'gaji_berkala',
        'maksimal',
        'status'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected $dates = [
        'deleted_at'
    ];

    protected $attributes = [ 
        'menuroles' => 'pegawai',
    ];

    public function instansi()
    {
        return $this->belongsTo(Instansi::class, 'instansi_id');
    }

    public static function pegawai()
    {
        return User::where('menuroles', 'pegawai');
    }

    public function riwayat()
    {
        return $this->hasMany(RiwayatPegawai::class);
    }

    public function agama()
    {
        return $this->belongsTo(Agama::class, 'religion_id');
    }

    public function getJabatanAttribute()
    {
        $jabatan = $this->riwayat->where('status', 1)->first()->jabatan;
        if($jabatan != null){
            $jabatan = $jabatan->name;
        } else {
            $jabatan = '-';
        }
        return $jabatan;
    }

    public function getPangkatAttribute()
    {
        $pangkat = $this->riwayat->where('status', 1)->first()->pangkat;
        if($pangkat != null){
            $pangkat = $pangkat->name;
        } else {
            $pangkat = '-';
        }
        return $pangkat;
    }

    public function salary_slip()
    {
        return $this->hasMany(SalarySlip::class);
    }
}
