@extends('dashboard.base')

@section('content')

        <div class="container-fluid">
          <div class="animated fadeIn">
            <div class="row justify-content-center">
              <div class="col-sm-12 col-md-8 col-lg-6">
                @if ($message = Session::get('fail'))
                  <div class="alert alert-danger alert-block">
                      <button type="button" class="close" data-dismiss="alert">×</button>    
                      <strong>{{ $message }}</strong>
                  </div>
                @endif
                <div class="card">
                    <div class="card-header">
                      <h4 class="m-0">Edit User</h4>
                    </div>
                    <div class="card-body">
                        <form method="POST" action="{{ route('users.update', $user->id) }}">
                            @csrf
                            @method('PUT')
                            <label for="">Nama</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">
                                    <svg class="c-icon c-icon-sm">
                                        <use xlink:href="/assets/icons/coreui/free-symbol-defs.svg#cui-user"></use>
                                    </svg>
                                  </span>
                              </div>
                              <input class="form-control" type="text" placeholder="{{ __('Nama') }}" name="name" value="{{ $user->name }}" required autofocus>
                            </div>
                            @error('name')
                              <div class="alert alert-danger py-1 px-2 mb-2">{{ $message }}</div>
                            @enderror
                            <label for="">Role</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">
                                    <svg class="c-icon c-icon-sm">
                                        <use xlink:href="/assets/icons/coreui/free-symbol-defs.svg#cui-view-module"></use>
                                    </svg>
                                  </span>
                              </div>
                              <select class="form-control" name="role" id="role" required>
                                <option selected disabled>Pilih salah satu</option>
                                @foreach ($roles as $role)
                                <option value="{{$role->id}}" @if ($user->menuroles == $role->name) selected @endif>{{$role->name}}</option>
                                  {{-- @if ($role->name != 'pegawai')
                                  @endif --}}
                                @endforeach
                              </select>
                            </div>
                            @error('role')
                              <div class="alert alert-danger py-1 px-2 mb-2">{{ $message }}</div>
                            @enderror
                            <label for="">Instansi</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">
                                    <svg class="c-icon c-icon-sm">
                                        <use xlink:href="/assets/icons/coreui/free-symbol-defs.svg#cui-bank"></use>
                                    </svg>
                                  </span>
                              </div>
                              <select class="form-control" name="instansi_id" id="instansi">
                                {{-- <option selected disabled>Pilih salah satu</option> --}}
                                <option value="0" selected>{{ env('APP_COMPANY','')}}</option>
                                @foreach ($instansis as $instansi)
                                <option value="{{$instansi->id}}" @if ($user->instansi_id == $instansi->id) selected @endif>{{$instansi->name}}</option>
                                @endforeach
                              </select>
                            </div>
                            <label for="">Email</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">@</span>
                              </div>
                              <input class="form-control" type="email" placeholder="{{ __('E-Mail') }}" name="email" value="{{ $user->email }}" required>
                            </div>
                            @error('email')
                              <div class="alert alert-danger py-1 px-2 mb-2">{{ $message }}</div>
                            @enderror
                            <label for="">Password</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">
                                    <svg class="c-icon c-icon-sm">
                                        <use xlink:href="/assets/icons/coreui/free-symbol-defs.svg#cui-lock-locked"></use>
                                    </svg>
                                  </span>
                              </div>
                              <input class="form-control" type="password" placeholder="{{ __('Masukkan password') }}" name="password">
                            </div>
                            @error('password')
                              <div class="alert alert-danger py-1 px-2 mb-2">{{ $message }}</div>
                            @enderror
                            <label for="">Konfirmasi Password</label>
                            <div class="input-group mb-3">
                              <div class="input-group-prepend">
                                  <span class="input-group-text">
                                    <svg class="c-icon c-icon-sm">
                                        <use xlink:href="/assets/icons/coreui/free-symbol-defs.svg#cui-lock-locked"></use>
                                    </svg>
                                  </span>
                              </div>
                              <input class="form-control" type="password" placeholder="{{ __('Konfirmasi password') }}" name="password_confirmation">
                            </div>
                            @error('password_confirmation')
                              <div class="alert alert-danger py-1 px-2 mb-2">{{ $message }}</div>
                            @enderror
                            <br>
                            <button class="btn btn-primary mr-1" type="submit">{{ __('Save') }}</button>
                            <a href="{{ route('users.index') }}" class="btn btn-outline-dark">{{ __('Back') }}</a> 
                        </form>
                    </div>
                </div>
              </div>
            </div>
          </div>
        </div>

@endsection

@section('javascript')
<script>
  $(document).ready(function() {
    $("#role").select2({placeholder: "Pilih role", allowClear: true});
    $("#instansi").select2({placeholder: "Pilih instansi", allowClear: true});
  });
</script>
@endsection