<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/
Route::get('/clear-cache', function() {
    Artisan::call('cache:clear');
    Artisan::call('config:clear');
    Artisan::call('view:clear');
    Artisan::call('route:clear');
    Artisan::call('config:cache');
    return "Cache is cleared";
});
Route::group(['middleware' => ['get.menu']], function () {
    Route::group(['middleware' => ['auth']], function () {
        Route::get('/', 'DashboardController@index');
        Route::get('/get-promo/{id}', 'DashboardController@getPromo');
    });
    Route::group(['middleware' => ['role:pegawai']], function () {
        Route::get('/404', function () {        return view('dashboard.404'); });
        Route::get('/500', function () {        return view('dashboard.500'); });
        Route::resource('notes', 'NotesController');
    });
    Auth::routes(['register' => false]);

    Route::resource('resource/{table}/resource', 'ResourceController')->names([
        'index'     => 'resource.index',
        'create'    => 'resource.create',
        'store'     => 'resource.store',
        'show'      => 'resource.show',
        'edit'      => 'resource.edit',
        'update'    => 'resource.update',
        'destroy'   => 'resource.destroy'
    ]);

    Route::group(['middleware' => ['auth']], function () {
        Route::resource('jabatan', 'JabatanController');
        Route::resource('pangkat', 'PangkatController');
        Route::resource('agama', 'AgamaController');
        Route::resource('jenis-potongan', 'JenisPotonganController');
        Route::resource('jenis-penambahan', 'JenisPenambahanController');
        Route::resource('pegawai', 'PegawaiController');
        Route::get('detail-pegawai', 'PegawaiController@detailPegawai')->name('pegawai.detail-pegawai');
        Route::get('pegawai/{id}/riwayat/create', 'PegawaiController@createRiwayat')->name('pegawai.riwayat.create');
        Route::post('pegawai/{id}/riwayat', 'PegawaiController@storeRiwayat')->name('pegawai.riwayat.store');
        Route::get('pegawai/riwayat/{id}/edit', 'PegawaiController@editRiwayat')->name('pegawai.riwayat.edit');
        Route::put('pegawai/riwayat/{id}', 'PegawaiController@updateRiwayat')->name('pegawai.riwayat.update');

        Route::get('daftar-potongan', 'AngsuranController@list')->name('potongan.instansi.list');
        Route::delete('daftar-potongan/{id}/delete', 'AngsuranController@destroy')->name('potongan.instansi.delete');
        Route::get('input-potongan', 'AngsuranController@index')->name('potongan.instansi.index');
        Route::post('input-potongan', 'AngsuranController@index')->name('potongan.instansi.upload');
        Route::post('input-potongan/store', 'AngsuranController@store')->name('potongan.instansi.store');
        
        Route::get('daftar-potongan-internal', 'PotonganInternalController@list')->name('potongan.internal.list');
        Route::delete('daftar-potongan-internal/{id}/delete', 'PotonganInternalController@destroy')->name('potongan.internal.delete');
        Route::get('input-potongan-internal', 'PotonganInternalController@index')->name('potongan.internal.index');
        Route::post('input-potongan-internal', 'PotonganInternalController@index')->name('potongan.internal.upload');
        Route::post('input-potongan-internal/store', 'PotonganInternalController@store')->name('potongan.internal.store');
        Route::get('potongan-internal/export-draft', 'PotonganInternalController@export')->name('potongan.internal.download-draft');

        // Route::get('daftar-penambahan', 'PenambahanInternalController@list')->name('penambahan.instansi.list');
        // Route::delete('daftar-penambahan/{id}/delete', 'PenambahanInternalController@destroy')->name('penambahan.instansi.delete');
        Route::get('daftar-penambahan-internal', 'PenambahanInternalController@list')->name('penambahan.internal.list');
        Route::delete('daftar-penambahan-internal/{id}/delete', 'PenambahanInternalController@destroy')->name('penambahan.internal.delete');
        Route::get('input-penambahan-internal', 'PenambahanInternalController@index')->name('penambahan.internal.index');
        Route::post('input-penambahan-internal', 'PenambahanInternalController@index')->name('penambahan.internal.upload');
        Route::post('input-penambahan-internal/store', 'PenambahanInternalController@store')->name('penambahan.internal.store');
        Route::get('penambahan-internal/export-draft', 'PenambahanInternalController@export')->name('penambahan.internal.download-draft');

        Route::resource('bread',  'BreadController');   //create BREAD (resource)
        Route::resource('users',        'UsersController');
        // Route::resource('users',        'UsersController')->except( ['create', 'store'] );
        Route::resource('roles',        'RolesController');
        Route::resource('mail',        'MailController');
        Route::get('prepareSend/{id}',        'MailController@prepareSend')->name('prepareSend');
        Route::post('mailSend/{id}',        'MailController@send')->name('mailSend');
        Route::get('/roles/move/move-up',      'RolesController@moveUp')->name('roles.up');
        Route::get('/roles/move/move-down',    'RolesController@moveDown')->name('roles.down');
        Route::prefix('menu/element')->group(function () { 
            Route::get('/',             'MenuElementController@index')->name('menu.index');
            Route::get('/move-up',      'MenuElementController@moveUp')->name('menu.up');
            Route::get('/move-down',    'MenuElementController@moveDown')->name('menu.down');
            Route::get('/create',       'MenuElementController@create')->name('menu.create');
            Route::post('/store',       'MenuElementController@store')->name('menu.store');
            Route::get('/get-parents',  'MenuElementController@getParents');
            Route::get('/edit',         'MenuElementController@edit')->name('menu.edit');
            Route::post('/update',      'MenuElementController@update')->name('menu.update');
            Route::get('/show',         'MenuElementController@show')->name('menu.show');
            Route::get('/delete',       'MenuElementController@delete')->name('menu.delete');
        });
        Route::prefix('menu/menu')->group(function () { 
            Route::get('/',         'MenuController@index')->name('menu.menu.index');
            Route::get('/create',   'MenuController@create')->name('menu.menu.create');
            Route::post('/store',   'MenuController@store')->name('menu.menu.store');
            Route::get('/edit',     'MenuController@edit')->name('menu.menu.edit');
            Route::post('/update',  'MenuController@update')->name('menu.menu.update');
            Route::get('/delete',   'MenuController@delete')->name('menu.menu.delete');
        });
        Route::prefix('media')->group(function () {
            Route::get('/',                 'MediaController@index')->name('media.folder.index');
            Route::get('/folder/store',     'MediaController@folderAdd')->name('media.folder.add');
            Route::post('/folder/update',   'MediaController@folderUpdate')->name('media.folder.update');
            Route::get('/folder',           'MediaController@folder')->name('media.folder');
            Route::post('/folder/move',     'MediaController@folderMove')->name('media.folder.move');
            Route::post('/folder/delete',   'MediaController@folderDelete')->name('media.folder.delete');;

            Route::post('/file/store',      'MediaController@fileAdd')->name('media.file.add');
            Route::get('/file',             'MediaController@file');
            Route::post('/file/delete',     'MediaController@fileDelete')->name('media.file.delete');
            Route::post('/file/update',     'MediaController@fileUpdate')->name('media.file.update');
            Route::post('/file/move',       'MediaController@fileMove')->name('media.file.move');
            Route::post('/file/cropp',      'MediaController@cropp');
            Route::get('/file/copy',        'MediaController@fileCopy')->name('media.file.copy');
        });

        Route::prefix('penerimaan')->group(function () { 
            Route::get('/', 'PenerimaanController@index')->name('penerimaan.index');
            Route::get('/generate', 'PenerimaanController@generate')->name('penerimaan.generate');
            Route::post('/store', 'PenerimaanController@store')->name('penerimaan.store');
            Route::get('/{id}/daftar', 'PenerimaanController@list')->name('penerimaan.list');
            Route::get('/{id}/edit-potongan', 'PenerimaanController@editPotongan')->name('penerimaan.edit-potongan');
            Route::put('/{id}/update-potongan', 'PenerimaanController@updatePotongan')->name('penerimaan.update-potongan');
            Route::get('/{id}/detail', 'PenerimaanController@show')->name('penerimaan.detail');
            Route::get('/{id}/cetak', 'PenerimaanController@print')->name('penerimaan.cetak');
            Route::delete('/{id}/delete', 'PenerimaanController@destroy')->name('penerimaan.delete-slip');
            Route::get('/download', 'PenerimaanController@download')->name('penerimaan.download');
            Route::get('/download-excel', 'PenerimaanController@downloadExcel')->name('penerimaan.download-excel');
            Route::get('/download-daftar', 'PenerimaanController@downloadDaftar')->name('penerimaan.download-daftar');
            Route::get('/agree/check', 'PenerimaanController@agreeCheck')->name('penerimaan.agree.check');
            Route::post('/setuju', 'PenerimaanController@agree')->name('penerimaan.agree');
            Route::post('/keberatan/{id}', 'PenerimaanController@keberatan')->name('penerimaan.keberatan');
        });

        Route::prefix('keuangan')->group(function () { 
            Route::get('/transaksi', 'KeuanganController@index')->name('keuangan.index');
            Route::get('/input-transaksi', 'KeuanganController@input')->name('keuangan.input');
            Route::post('/store', 'KeuanganController@store')->name('keuangan.store');
            Route::delete('/{id}/delete', 'KeuanganController@destroy')->name('keuangan.delete');
            Route::get('/{id}/edit', 'KeuanganController@edit')->name('keuangan.edit');
            Route::post('/update', 'KeuanganController@update')->name('keuangan.update');
            Route::get('/laporan', 'KeuanganController@report')->name('keuangan.report');
            Route::post('/laporan/generate', 'KeuanganController@generateReport')->name('keuangan.report.generate');
            Route::get('/laporan/print', 'KeuanganController@print')->name('keuangan.report.print');
        });

        Route::resource('promo', 'PromoController');
        Route::resource('permissions', 'PermissionsController');
        Route::resource('instansi', 'InstansiController');
        Route::get('profil', 'ProfilController@show')->name('profil.show');
        Route::put('profil/{id}', 'ProfilController@update')->name('profil.update');
        Route::get('backup-restore', 'BackupController@index');
        Route::get('backup-restore/backup', 'BackupController@backup');
        Route::post('backup-restore/restore', 'BackupController@restore')->name('admin.db.restore');

        // Route::group(['middleware' => ['role:instansi']], function () {
        //     Route::resource('promo', 'PromoController');
        // });
    });
});